/*
Copyright (c) AppDynamics, Inc., and its affiliates
2015
All Rights Reserved
*/
'use strict';


var os = require('os');


/*
 * Sending process related data as metrics.
 */

function ProcessStats(agent) {
  this.agent = agent;

  this.lastCpuTime = undefined;
}
exports.ProcessStats = ProcessStats;


ProcessStats.prototype.init = function() {
  var self = this;

  self.agent.metricsManager.addMetric('Process', 'Node RSS',
    function() { return process.memoryUsage().rss / 1048576; },
    'MB', 'set');
  self.agent.metricsManager.addMetric('Process', 'V8 heap used',
    function() { return process.memoryUsage().heapUsed / 1048576; },
    'MB', 'set');

  self.lastTimestamp = Date.now();
  self.lastCpuTime = self.agent.system.cputime() || 0;
  self.agent.metricsManager.addMetric('Process', 'CPU usage', function() {
    var timeNow = Date.now();
    var cpuTime = self.agent.system.cputime() || 0;
    var result = ((cpuTime - self.lastCpuTime) / 1000) / (timeNow - self.lastTimestamp) * 100;

    self.lastTimestamp = timeNow;
    self.lastCpuTime = cpuTime;
    return result;
  }, 'ms', 'set');
};
